@@ -2,7 +2,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   Catfish - a versatile file searching tool
 #   Copyright (C) 2007-2012 Christian Dywan <christian@twotoasts.de>
-#   Copyright (C) 2012-2016 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2018 Sean Davis <smd.seandavis@gmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
 #   under the terms of the GNU General Public License version 2, as published
@@ -122,7 +122,7 @@ class CatfishSearchEngine:
         """Add a CatfishSearchMethod the the engine's search backends."""
         self.methods.append(method_class())
 
-    def run(self, keywords, path, limit=-1, regex=False):
+    def run(self, keywords, path, limit=-1, regex=False): # noqa
         """Run the CatfishSearchEngine.
 
         Each method is run sequentially in the order they are added.
@@ -131,6 +131,8 @@ class CatfishSearchEngine:
         filename is yielded if it matches the query.  False is also yielded
         afterwards to guarantee the interface does not lock up."""
         self.start_time = time.time()
+        self.stop_time = 0
+
         keywords = keywords.replace(',', ' ').strip().lower()
 
         logger.debug("[%i] path: %s, keywords: %s, limit: %i, regex: %s",
@@ -155,13 +157,36 @@ class CatfishSearchEngine:
         for key in keywords.split():
             keys.append(key.strip())
 
+        # Path exclusions for efficiency
+        exclude = []
+        cache_path = os.path.expanduser("~/.cache")
+        if cache_path not in path:
+            exclude.append(cache_path)
+        gvfs_path = os.path.expanduser("~/.gvfs")
+        if gvfs_path not in path:
+            exclude.append(gvfs_path)
+
         file_count = 0
         for method in self.methods:
+            if self.stop_time > 0:
+                logger.debug("Engine is stopped")
+                return
             logger.debug(
                 "[%i] Starting search method: %s",
                 self.engine_id, method.method_name)
             for filename in method.run(keywords, path, regex):
                 if isinstance(filename, str) and path in filename:
+                    found_bad = False
+                    for filepath in exclude:
+                        if filepath in filename:
+                            if self.stop_time > 0:
+                                logger.debug("Engine is stopped")
+                                return
+                            found_bad = True
+                    if found_bad:
+                        yield True
+                        continue
+
                     if method.method_name == 'fulltext' or  \
                             all(key in
                                 os.path.basename(filename).lower()
@@ -264,10 +289,19 @@ class CatfishSearchMethod_Walk(CatfishSearchMethod):
 
         This function is a generator and will yield files as they are found or
         True if still running."""
+        exclude = []
+        cache_path = os.path.expanduser("~/.cache")
+        if cache_path not in path:
+            exclude.append(cache_path)
+        gvfs_path = os.path.expanduser("~/.gvfs")
+        if gvfs_path not in path:
+            exclude.append(gvfs_path)
+
         self.running = True
         if isinstance(keywords, str):
             keywords = keywords.replace(',', ' ').strip().split()
         for root, dirs, files in os.walk(path, False):
+            dirs[:] = [d for d in dirs if os.path.join(root, d) not in exclude]
             if not self.running:
                 break
             paths = dirs + files
@@ -300,7 +334,7 @@ class CatfishSearchMethod_Fulltext(CatfishSearchMethod):
         self.running = False
         self.exact = False
 
-    def run(self, keywords, path, regex=False):
+    def run(self, keywords, path, regex=False): # noqa
         """Run the search method using keywords and path.  regex is not used
         by this search method.
 
@@ -468,7 +502,7 @@ class CatfishSearchMethodExternal(CatfishSearchMethod):
     def process_output(self, output):
         """Return the output text."""
         if isinstance(output, io.BufferedReader):
-            return map(lambda s: s.decode(encoding='UTF8').strip(),
+            return map(lambda s: s.decode(encoding='UTF8', errors='replace').strip(),
                        output.readlines())
         else:
             return output
@@ -507,5 +541,5 @@ class CatfishSearchMethod_Locate(CatfishSearchMethodExternal):
 
     def assemble_query(self, keywords, path):
         """Assemble the search query."""
-        return ["locate", "--regex", "-i", "{}".format(string_regex(keywords,
-                                                                    path))]
+        return ["locate", "--regex", "--basename", "-i",
+                "{}".format(string_regex(keywords, path))]
